from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.http import HttpResponseBadRequest, HttpResponseServerError, HttpResponse

from .models import WebMentionResponse
from .resolution import url_resolves, fetch_and_validate_source, SourceFetchError, TargetNotFoundError


@csrf_exempt
@require_POST
def receive(request):
    if 'source' in request.POST and 'target' in request.POST:
        source = request.POST.get('source')
        target = request.POST.get('target')
        webmention = None

        if not url_resolves(target):
            return HttpResponseBadRequest('Target URL did not resolve to a resource on the server')

        try:
            try:
                webmention = WebMentionResponse.objects.get(source=source, response_to=target)
            except WebMentionResponse.DoesNotExist:
                webmention = WebMentionResponse()

            response_body = fetch_and_validate_source(source, target)
            webmention.update(source, target, response_body)
            return HttpResponse(status_code=202, reason_phrase='The webmention was successfully received')
        except (SourceFetchError, TargetNotFoundError) as e:
            webmention.invalidate()
            return HttpResponseBadRequest(str(e))
        except Exception as e:
            return HttpResponseServerError(str(e))
    else:
        return HttpResponseBadRequest('webmention source and/or target not in request')




from unittest.mock import Mock, patch
from django.core.urlresolvers import Resolver404

from django.test import TestCase

from ..resolution import url_resolves, fetch_and_validate_source, SourceFetchError, TargetNotFoundError


class ResolutionTestCase(TestCase):
    def setUp(self):
        self.source = 'http://example.com'
        self.target = 'http://mysite.com'

    @patch('webmention.resolution.resolve')
    def test_url_resolves_when_resolves(self, mock_resolve):
        mock_resolve.return_value = 'foo'
        self.assertTrue(url_resolves(self.target))

    @patch('webmention.resolution.resolve')
    def test_url_resolves_when_does_not_resolve(self, mock_resolve):
        mock_resolve.side_effect = Resolver404
        self.assertFalse(url_resolves('http://example.com/page'))

    @patch('requests.get')
    def test_fetch_and_validate_source_happy_path(self, mock_get):
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.content = '<a href="{href}">{href}</a>'.format(href=self.target)
        mock_get.return_value = mock_response

        self.assertEqual(mock_response.content, fetch_and_validate_source(self.source, self.target))

    @patch('requests.get')
    def test_fetch_and_validate_source_when_source_unavailable(self, mock_get):
        mock_response = Mock()
        mock_response.status_code = 404
        mock_get.return_value = mock_response

        self.assertRaises(SourceFetchError, fetch_and_validate_source, self.source, self.target)

    @patch('requests.get')
    def test_fetch_and_validate_source_when_source_does_not_contain_target(self, mock_get):
        mock_response = Mock()
        mock_response.status_code = 200
        mock_response.content = 'foo'
        mock_get.return_value = mock_response

        self.assertRaises(TargetNotFoundError, fetch_and_validate_source, self.source, self.target)



from unittest.mock import patch

from django.test import TestCase

from webmention.models import WebMentionResponse


class WebMentionResponseTestCase(TestCase):
    def setUp(self):
        self.source = 'http://example.com'
        self.target = 'http://mysite.com'
        self.response_body = 'foo'

    def test_str(self):
        webmention = WebMentionResponse.objects.create(source=self.source, response_to=self.target, response_body=self.response_body)
        webmention.save()

        self.assertEqual(webmention.source, str(webmention))

    def test_source_for_admin(self):
        webmention = WebMentionResponse.objects.create(source=self.source, response_to=self.target, response_body=self.response_body)
        webmention.save()

        self.assertEqual('<a href="{href}">{href}</a>'.format(href=webmention.source), webmention.source_for_admin())

    def test_response_to_for_admin(self):
        webmention = WebMentionResponse.objects.create(source=self.source, response_to=self.target, response_body=self.response_body)
        webmention.save()

        self.assertEqual('<a href="{href}">{href}</a>'.format(href=webmention.response_to), webmention.response_to_for_admin())

    @patch('webmention.models.WebMentionResponse.save')
    def test_invalidate_when_not_previously_saved(self, mock_save):
        webmention = WebMentionResponse()
        webmention.invalidate()

        self.assertFalse(mock_save.called)

    def test_invalidate_when_previously_saved(self):
        webmention = WebMentionResponse.objects.create(source=self.source, response_to=self.target, response_body=self.response_body)
        webmention.save()
        webmention.invalidate()

        self.assertFalse(webmention.current)

    @patch('webmention.models.WebMentionResponse.save')
    def test_update_when_previously_invalid(self, mock_save):
        webmention = WebMentionResponse.objects.create(source='foo', response_to='bar', response_body='baz', current=False)
        self.assertEqual(1, mock_save.call_count)
        webmention.update(self.source, self.target, self.response_body)

        self.assertTrue(webmention.current)
        self.assertEqual(self.source, webmention.source)
        self.assertEqual(self.target, webmention.response_to)
        self.assertEqual(self.response_body, webmention.response_body)
        self.assertEqual(2, mock_save.call_count)



from django.db import models


class WebMentionResponse(models.Model):
    response_body = models.TextField()
    response_to = models.URLField()
    source = models.URLField()
    reviewed = models.BooleanField(default=False)
    current = models.BooleanField(default=True)
    date_created = models.DateTimeField(auto_now_add=True)
    date_modified = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name = 'webmention'
        verbose_name_plural = 'webmentions'

    def __str__(self):
        return self.source

    def source_for_admin(self):
        return '<a href="{href}">{href}</a>'.format(href=self.source)
    source_for_admin.allow_tags = True
    source_for_admin.short_description = 'source'

    def response_to_for_admin(self):
        return '<a href="{href}">{href}</a>'.format(href=self.response_to)
    response_to_for_admin.allow_tags = True
    response_to_for_admin.short_description = 'response to'

    def invalidate(self):
        if self.id:
            self.current = False
            self.save()

    def update(self, source, target, response_body):
        self.response_body = response_body
        self.source = source
        self.response_to = target
        self.current = True
        self.save()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='WebMentionResponse',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, primary_key=True, serialize=False)),
                ('response_body', models.TextField()),
                ('response_to', models.URLField()),
                ('source', models.URLField()),
                ('reviewed', models.BooleanField(default=False)),
                ('current', models.BooleanField(default=True)),
                ('date_created', models.DateTimeField(auto_now_add=True)),
                ('date_modified', models.DateTimeField(auto_now=True)),
            ],
            options={
                'verbose_name': 'webmention',
                'verbose_name_plural': 'webmentions',
            },
        ),
    ]



import requests

from urllib.parse import urlparse
from django.core.urlresolvers import resolve, Resolver404


def url_resolves(url):
    try:
        resolve(urlparse(url).path)
    except Resolver404:
        return False
    return True


def fetch_and_validate_source(source, target):
    response = requests.get(source)
    if response.status_code == 200:
        if target in str(response.content):
            return response.content
        else:
            raise TargetNotFoundError('Source URL did not contain target URL')
    else:
        raise SourceFetchError('Could not fetch source URL')


class SourceFetchError(Exception):
    pass


class TargetNotFoundError(Exception):
    pass



from django.core.urlresolvers import reverse
from django.utils.decorators import decorator_from_middleware


class WebMentionMiddleware(object):
    def process_response(self, request, response):
        link_header = '<{scheme}://{host}{path}>; rel="webmention"'.format(
            scheme=request.scheme,
            host=request.META.get('HTTP_HOST'),
            path=reverse('webmention:receive')
        )
        if not response.get('Link'):
            response['Link'] = link_header
        else:
            response['Link'] = ', '.join((response['Link'], link_header))

        return response

include_webmention_information = decorator_from_middleware(WebMentionMiddleware)



from unittest.mock import Mock, patch

from django.test import TestCase
from django.http import HttpResponseBadRequest, HttpResponse, HttpResponseServerError

from ..views import receive
from ..resolution import SourceFetchError, TargetNotFoundError


class ReceiveTestCase(TestCase):
    def setUp(self):
        self.source = 'http://example.com'
        self.target = 'http://mysite.com'

    def test_receive_when_source_not_in_post_data(self):
        request = Mock()
        request.method = 'POST'
        request.POST = {'target': self.target}

        response = receive(request)

        self.assertTrue(isinstance(response, HttpResponseBadRequest))

    def test_receive_when_target_not_in_post_data(self):
        request = Mock()
        request.method = 'POST'
        request.POST = {'source': self.source}

        response = receive(request)

        self.assertTrue(isinstance(response, HttpResponseBadRequest))

    @patch('webmention.views.url_resolves')
    def test_receive_when_target_does_not_resolve(self, mock_url_resolves):
        request = Mock()
        request.method = 'POST'
        request.POST = {'source': self.source, 'target': self.target}

        mock_url_resolves.return_value = False
        response = receive(request)

        mock_url_resolves.assert_called_once_with(self.target)
        self.assertTrue(isinstance(response, HttpResponseBadRequest))

    @patch('webmention.views.WebMentionResponse.update')
    @patch('webmention.views.fetch_and_validate_source')
    @patch('webmention.views.url_resolves')
    def test_receive_happy_path(self, mock_url_resolves, mock_fetch_and_validate_source, mock_update):
        request = Mock()
        request.method = 'POST'
        request.POST = {'source': self.source, 'target': self.target}

        mock_url_resolves.return_value = True
        mock_fetch_and_validate_source.return_value = 'foo'
        response = receive(request)

        mock_fetch_and_validate_source.assert_called_once_with(self.source, self.target)
        mock_update.assert_called_once_with(self.source, self.target, mock_fetch_and_validate_source.return_value)
        mock_url_resolves.assert_called_once_with(self.target)
        self.assertTrue(isinstance(response, HttpResponse))

    @patch('webmention.views.WebMentionResponse.invalidate')
    @patch('webmention.views.fetch_and_validate_source')
    @patch('webmention.views.url_resolves')
    def test_receive_when_source_unavailable(self, mock_url_resolves, mock_fetch_and_validate_source, mock_invalidate):
        request = Mock()
        request.method = 'POST'
        request.POST = {'source': self.source, 'target': self.target}

        mock_url_resolves.return_value = True
        mock_fetch_and_validate_source.side_effect = SourceFetchError
        response = receive(request)

        mock_fetch_and_validate_source.assert_called_once_with(self.source, self.target)
        mock_url_resolves.assert_called_once_with(self.target)
        self.assertEqual(1, mock_invalidate.call_count)
        self.assertTrue(isinstance(response, HttpResponseBadRequest))

    @patch('webmention.views.WebMentionResponse.invalidate')
    @patch('webmention.views.fetch_and_validate_source')
    @patch('webmention.views.url_resolves')
    def test_receive_when_source_does_not_contain_target(self, mock_url_resolves, mock_fetch_and_validate_source, mock_invalidate):
        request = Mock()
        request.method = 'POST'
        request.POST = {'source': self.source, 'target': self.target}

        mock_url_resolves.return_value = True
        mock_fetch_and_validate_source.side_effect = TargetNotFoundError
        response = receive(request)

        mock_fetch_and_validate_source.assert_called_once_with(self.source, self.target)
        mock_url_resolves.assert_called_once_with(self.target)
        self.assertEqual(1, mock_invalidate.call_count)
        self.assertTrue(isinstance(response, HttpResponseBadRequest))

    @patch('webmention.views.fetch_and_validate_source')
    @patch('webmention.views.url_resolves')
    def test_receive_when_general_exception_occurs(self, mock_url_resolves, mock_fetch_and_validate_source):
        request = Mock()
        request.method = 'POST'
        request.POST = {'source': self.source, 'target': self.target}

        mock_url_resolves.return_value = True
        mock_fetch_and_validate_source.side_effect = Exception
        response = receive(request)

        mock_fetch_and_validate_source.assert_called_once_with(self.source, self.target)
        mock_url_resolves.assert_called_once_with(self.target)
        self.assertTrue(isinstance(response, HttpResponseServerError))



from unittest.mock import Mock
from django.core.urlresolvers import reverse

from django.http import HttpResponse
from django.test import TestCase, Client

from ..middleware import WebMentionMiddleware


class WebMentionMiddlewareTestCase(TestCase):
    def setUp(self):
        self.client = Client()
        self.middleware = WebMentionMiddleware()

    def test_process_request_creates_link_header(self):
        request = Mock()
        request.scheme = 'http'
        request.META = {'HTTP_HOST': 'example.com'}

        response = HttpResponse()
        response = self.middleware.process_response(request, response)

        expected_link_header = '<{scheme}://{host}{path}>; rel="webmention"'.format(
            scheme=request.scheme,
            host=request.META.get('HTTP_HOST'),
            path=reverse('webmention:receive')
        )

        self.assertIn('Link', response)
        self.assertEqual(expected_link_header, response['Link'])

    def test_process_request_appends_link_header(self):
        request = Mock()
        request.scheme = 'http'
        request.META = {'HTTP_HOST': 'example.com'}

        response = HttpResponse()
        original_link_header = '<meta.rdf>; rel="meta"'
        response['Link'] = original_link_header
        response = self.middleware.process_response(request, response)

        new_link_header = '<{scheme}://{host}{path}>; rel="webmention"'.format(
            scheme=request.scheme,
            host=request.META.get('HTTP_HOST'),
            path=reverse('webmention:receive')
        )

        expected_link_header = ', '.join((original_link_header, new_link_header))

        self.assertIn('Link', response)
        self.assertEqual(expected_link_header, response['Link'])



SECRET_KEY = 'key-for-testing'
INSTALLED_APPS = [
    'webmention',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'tests.sqlite3',
    }
}

ROOT_URLCONF = 'webmention.tests.test_urls'



from django.contrib import admin

from .models import WebMentionResponse


class WebMentionResponseAdmin(admin.ModelAdmin):
    model = WebMentionResponse

    fields = [
        (
            'source_for_admin',
            'response_to_for_admin',
        ),
        'response_body',
        (
            'date_created',
            'date_modified',
        ),
        (
            'reviewed',
            'current',
        )
    ]

    readonly_fields = [
        'response_body',
        'source_for_admin',
        'response_to_for_admin',
        'date_created',
        'date_modified',
        'current',
    ]

    list_display = [
        'pk',
        'source_for_admin',
        'response_to_for_admin',
        'date_created',
        'date_modified',
        'reviewed',
        'current',
    ]

    list_editable = [
        'reviewed',
    ]

    list_filter = [
        'reviewed',
        'current',
    ]

    date_hierarchy = 'date_modified'

admin.site.register(WebMentionResponse, WebMentionResponseAdmin)



import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
        readme = readme.read()

with open(os.path.join(os.path.dirname(__file__), 'requirements.txt')) as requirements_file:
    requirements = [line.rstrip() for line in requirements_file if line != '\n']

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-webmention',
    version='0.0.4',
    packages=find_packages(),
    description='A pluggable implementation of webmention for Django projects.',
    keywords='webmention pingback linkback blogging',
    author='Dane Hillard',
    author_email='github@danehillard.com',
    long_description=readme,
    install_requires=requirements,
    url='https://github.com/daneah/django-webmention',
    license='MIT',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
